//
/* This Program will open a netCDF file and create and write to a GEMPAK grid 
 * File
 */
// C. Bailey/HPC		10/2009		Creation
// C. Bailey/WPC                01/2023         Fix strncpy to strcpy

#include <string.h>
#include <netcdf.h>
#include "geminc.h"
#include "gemprm.h"
#include "nc2grid.h"

void nc_getreftime (int ncid, int nvars, int ritarr[5], ncvar_t var, int *iret);
void nc_getvalidtimes (int ncid, ncdim_t *dims, int nvars, int ritarr[5], ncvar_t var, int timeflg, char valtimearray[][16], int *iret);
void nc_getgridnav (int ncid, ncdim_t *dims, int ndims, ncgatt_t *gatts, 
                    int ngatts, ncvar_t var, int nvars, char *cproj, int *kx, 
                    int *ky, float *gdsarr, float *corners, float *navblk, 
                    int *ntimes, int *ncfiletype, int *iret);
void nc_getvarlevels (int ncid, int nlevels, ncvar_t *var, int nvars, int varid,
                      int *ilevels1, int *ilevels2, char **coord,
                      int *iret);
void nc_wxstrings (int ncid, int nvars, char *wxstringname, ncdim_t *dims, 
                   char valtimearray[][16], int *iret);

int main ( int argc, char **argv )
{
	char cdf_file;
	int ncid; // pointer to location netCDF file ID is stored
	int nc_status;

	ncdim_t *dims;  // File dimensions definitions
        ncvar_t var;    // variable definitions
        ncatt_t att;    // variable attributes definitions
        ncgatt_t *gatts; // Global attibute structure

	int ndims;      // pointer to location for returned # of dimensions
        int nvars;      // pointer to location for returned # of variables 
        int ngatts;     // pointer to location for returned # of global attributes
        int unlimdimid; 

	size_t length, recs, att_len;
        int dimid, recid, varid, attnum, iret;
	char recname[NC_MAX_NAME+1], varname[NC_MAX_NAME+1], attname[NC_MAX_NAME+1];
        nc_type var_type, att_type;
        int var_ndims, var_dims[NC_MAX_VAR_DIMS], var_natts;
        int    termflg, fileflg, nfps, prttl;
	int    navsiz, gsflag, igxold, iprec=5;
	float *var_val;
	short *svar_val;
	double *dvar_val;
        int*ivar_val;
	signed char *sc_val;
	double *dreftime;
	int itarr[5];

	char gdoutf[LLMXLN], proj[LLMXLN], grdarea[LLMXLN], kxky[LLMXLN];
        char cpyfil[LLMXLN], garea[LLMXLN];
        char   tmpproj[24], outfil[LLMXLN], vparm[12];
        int fill, maxgrid, igdfln, igrdnm, igx, igy, jx, jy, ksbx[2], ksby[2], subset, ihd, ii, timesinarray;
        float  rnvblk[LLNNAV]; 

	char **coord;
	int *ilevels1;
	int *ilevels2;

	float gdsarr[10];
	float corners[4];
	float navblk[LLNNAV];
	int kx,ky, skip, ntimes, mode=1, done=0;
	int inx, iny, replace=0, ighdr[LLGDHD], pkmeth=MDGDEC;
	char cproj[4];
	int respnd, i, ier, iperr, err, ncfiletype, strlength;
	nc_input ui;
	char   *gplt_dev="GN";
	FILE *fps[MMFILE], *fp;
/*----------------------------------------------------------------------*/
    /*
     *  Initialize TAE.
     */
    ip_init ( &respnd, &iperr );
    if ( iperr == 0 ) 
        ip_idnt ( "NC2GRID", &done, strlen("NC2GRID") );
	
    /*
     *  Initialize the GEMPAK plotting package.
     */
    gg_init ( &mode, &iret );
    if ( iret != 0 ) exit(1);

    /*
     *  Initialize grid library common area grdcmn.cmn.
     */
    gd_init ( &iret );

/*
 *  Set device.
 */
    gg_sdev(gplt_dev, &iret, sizeof(gplt_dev) );

    skip = 1;
    while ( done == 0 ) {
	for ( i=0; i<LLGDHD; i++ ) ighdr[i]=0;
/*
 *  Wait for user input.
 */
	if ( skip == 0 ) ip_dynm( &done, &iret );
	skip = 0;
	if ( done != 0 ) break;          /*  Exit out of interactive loop  */
	
/*
 *  Get the user input.
 */
        ncginp ( &ui, &iperr );
        if ( iperr != 0 ) {
            er_wmsg("NC2GRID",&iperr," ",&ier,7,1);
        }
        igdfln=0;
/*
 *  Open Output devices.
 */
        inc_outt ( ui.output, "NC2GRID.fil", &termflg, &fileflg, outfil,
                   &ier );
        nfps = 0;
        if ( termflg == G_TRUE ) {
            fps[nfps++] = stdout;
        }
        if ( fileflg == G_TRUE ) {
            fp = cfl_wopn ( outfil, &ier );
            fps[nfps++] = fp;
        }

        /* OPEN netCDF file */
	nc_status = nc_open (ui.ncfile, NC_NOWRITE, &ncid);
        if (nc_status != NC_NOERR) {
	   err=-15;
	   er_wmsg("NC2GRID",&err," ",&ier,7,1);
	   continue;
	}
        /* Get number of dimensions, number of variables, number of global
         * attributes
	 */
	nc_status = nc_inq(ncid, &ndims, &nvars, &ngatts, &unlimdimid);
        /*
         * Load in file dimensions 
         */
	dims = (ncdim_t *) malloc((ndims + 1) * sizeof(ncdim_t));

	for (dimid = 0; dimid < ndims; dimid++ ) {
	    nc_status = nc_inq_dim (ncid, dimid, dims[dimid].name, &dims[dimid].size);
	}
        /*
	 * Load in Global Attributes
	 */
	gatts = (ncgatt_t *) malloc((ngatts + 1) * sizeof(ncgatt_t));
	int gattid;
	for (gattid = 0; gattid < ngatts; gattid++) {
	    nc_status = nc_inq_attname(ncid, NC_GLOBAL, gattid, gatts[gattid].name);
	    nc_status = nc_inq_att(ncid, NC_GLOBAL, gatts[gattid].name, &gatts[gattid].type, &gatts[gattid].size);
	    if (gatts[gattid].type == NC_CHAR) {
		gatts[gattid].string = (char *) malloc(gatts[gattid].size+1);
		nc_status = nc_get_att_text(ncid, NC_GLOBAL, gatts[gattid].name,  gatts[gattid].string);
	    } else if (gatts[gattid].type == NC_FLOAT) {
		gatts[gattid].fvals = (float *) malloc(gatts[gattid].size * sizeof(float));
		nc_status = nc_get_att_float(ncid, NC_GLOBAL, gatts[gattid].name,  gatts[gattid].fvals);
	    } else if (gatts[gattid].type == NC_INT) {
		gatts[gattid].ivals = (int *) malloc(gatts[gattid].size * sizeof(int));
                nc_status = nc_get_att_int(ncid, NC_GLOBAL, gatts[gattid].name,  gatts[gattid].ivals);
	    } else if (gatts[gattid].type == NC_DOUBLE) {
                gatts[gattid].dvals = (double *) malloc(gatts[gattid].size * sizeof(double));
                nc_status = nc_get_att_double(ncid, NC_GLOBAL, gatts[gattid].name,  gatts[gattid].dvals);
            }
	}
/*
 * 	Determine Grid dimensions, Projection, x, y, corner points
 */

	/* Get Grid navagation information and return GEMPAK navagation block */
	nc_getgridnav (ncid, dims, ndims, gatts, ngatts, var, nvars, cproj, &kx, &ky, gdsarr, corners, navblk, &ntimes, &ncfiletype,  &err);
	if (err != 0) {
            er_wmsg("NC2GRID",&err," ",&iret,7,1);
            break;
        }

	/* Open gempak grid file */

	ihd = 10;
	igdfln = 0;
	strcpy (gdoutf, ui.gdoutf);
	sprintf(proj, "%s/%4.2f;%4.2f;%4.2f", cproj, gdsarr[7], gdsarr[8], gdsarr[9]);
	sprintf(grdarea, "%4.3f;%4.3f;%4.3f;%4.3f", gdsarr[3], gdsarr[4], gdsarr[5], gdsarr[6]);
	sprintf(kxky, "%i;%i", kx, ky);
	strcpy(cpyfil, "");
	strcpy(garea, "GRID");
	maxgrid = 5000;
	fill = 0;
	int navsz = LLNNAV;
	int anlsz = LLNANL;
	float anlblk[LLNANL];

	for ( ii = 0; ii < LLNANL; ii++ ) {
            anlblk[ii] = 0.;
        }

/*
 * Find The Reference time, create array of times in YYYYMMDDHH format,
 * find Time units.
 */

	if (ntimes == -1) ntimes = 500;

	// Find time info in variables
	char refdate[15];
	char valtimearray[ntimes][16];
        int tdiffarray[ntimes];
	int ritarr[5];

	int novalidtime = 0, timeflg = 1;
	int i;

	nc_getreftime(ncid, nvars, ritarr, var, &timeflg);

	//printf ("timeflg = %i \n", timeflg);
	//printf ("ncfiletype = %i \n", ncfiletype);
	if (timeflg >= 0) {
	    nc_getvalidtimes (ncid, dims, nvars, ritarr, var, timeflg, valtimearray, &novalidtime);
            // Initialize tdiffarray to 0 if grabbing validtime from other files other than AWIPS
            //printf ("nimes = %i \n", ntimes);
            for ( i = 0; i < ntimes; i++) {
		 //printf("validTimeArray[%i] = %s \n", i, valtimearray[i]);
                 tdiffarray[i] = 0;
            }
	} else {
	    if ( ncfiletype == 6 ) {
		 nc_getvalidtimes (ncid, dims, nvars, ritarr, var, timeflg, valtimearray, &novalidtime);
	    } else {
	    novalidtime = 1;
	    }
	}

	//printf("++++ POST get Valid Tine ++++\n");

	/* Get Variable information */

	char parm[12], units[5];
	//ncvarinfo_t *varinfo;
	int nlevels;
	int griddims[10];
	int count=0;
/*
 *	Loop thru all variables in netCDF
 */
	if (nvars > 0) {
            for (varid = 0; varid < nvars; varid++ ) {
/*
 *              Get variable Information (name, type, shapes)
 */
		nc_status = nc_inq_varndims(ncid, varid, &var.ndims);
		var.dims = (int *) malloc((var.ndims + 1) * sizeof(int));
		nc_status = nc_inq_var (ncid, varid, var.name, &var.type, 0, var.dims, &var.natts);

/*
 *		Read NetCDF parameter file, match file contents to current
 *		variable. Decode and save in file if variable matches parameter
 *		in table file.
 */
		int proces;
                N2Gvars_t *ncvartbl;
                N2GVinfo ncvar;
		float scale_factor = 1.0;
		float add_offset = 0.0;
		float missing_value;

		proces = 1;
                nc_gtvartbl(ui.nctbls, &ncvartbl, &ier);

                if (ier == 0) {
                    nc_skvar(var.name, ncvartbl, &ncvar, &ier);
                }
		if (ier != 0) {
		    err = 1;
		    er_wmsg("NC2GRID",&err,var.name,&iret,7,strlen(var.name));
		    proces = 0;
		}
		if (proces == 1) {
		    /*
		     * Get # of Levels from variable dimensions.
		     */
		    for ( i = 0; i < var.ndims; i++) {
		        griddims[i] = dims[var.dims[i]].size;
		        if (strcmp(dims[var.dims[i]].name,"record") == 0 || strcmp(dims[var.dims[i]].name,"time") == 0) {
		        } else if (strcmp(dims[var.dims[i]].name,"levels_1")==0 || 
			           strcmp(dims[var.dims[i]].name,"levels_2")==0 ||
			           strcmp(dims[var.dims[i]].name,"levels_3")==0 ||
			           strcmp(dims[var.dims[i]].name,"levels_4")==0 ||
			           strcmp(dims[var.dims[i]].name,"levels_5")==0 ||
			           strcmp(dims[var.dims[i]].name,"levels_6")==0 ||
                                   strcmp(dims[var.dims[i]].name,"levels_7")==0 ||
                                   strcmp(dims[var.dims[i]].name,"levels_8")==0 ||
			           strcmp(dims[var.dims[i]].name,"levels_9")==0 ||
                                   strcmp(dims[var.dims[i]].name,"levels_10")==0 ||
                                   strcmp(dims[var.dims[i]].name,"levels_11")==0 ||
                                   strcmp(dims[var.dims[i]].name,"levels_12")==0 ||
				   strcmp(dims[var.dims[i]].name,"level") == 0) {
			    nlevels = dims[var.dims[i]].size;
			    break;
		        } else {
			    nlevels = 1;
		        }
		    }
		
		    //printf ("name = %s, id = %i, type = %i, ndims = %i, natts = %i \n", var.name, varid, var.type, var.ndims,var.natts);

		    int j;
		    /*for (j = 0; j < var.ndims; j++) {
		        printf ("var dim %i = %i %s \n", j, dims[var.dims[j]].size, dims[var.dims[j]].name);
		    }*/
		}
                int tdiff;
                tdiff = 0;
             
		/*
		 * Find Attribute information for current variable
		 */
		if (proces == 1) {
		    /*
                     * Find validTimes Attribute if not found in NetCDF global 
                     * attributes 
                     */
		    if (novalidtime == 1) {
		        for (attnum = 0; attnum < var.natts; attnum++) {
                            tdiff = 0;
		            nc_status = nc_inq_attname (ncid, varid, attnum, att.name);
			    // Get variable attributes
		            nc_status = nc_inq_att (ncid, varid, att.name, &att.type, &att.len);
			    if (strcmp(att.name,"validTimes") == 0 ) {
			        if (att.type <= NC_INT) {
                                    int imin, jmin, itarr[5], jtarr[5], zz, timediff;
                                    att.ivals = (int *) malloc(att.len * sizeof(int));
                                    nc_status = nc_get_att_int (ncid, varid, att.name, att.ivals);
                                    for ( i = 0; i<(att.len/2); i++) {
				        zz = (i*2)+1-ncvar.timeflg;
					//printf("timeflg = %i, i = %i \n zz = %i \n", ncvar.timeflg, i, zz);
                                        imin = att.ivals[zz]/60;
					if (ncvar.timeflg == 0 ) {
                                            jmin = att.ivals[zz-1]/60;
                                            tdiff = (imin - jmin)/60;
                                        } else {
                                            jmin = att.ivals[zz+1]/60;
                                            tdiff = (jmin - imin)/60;
                                        }
                                       
                                        //printf("imin = %i\n", imin);
                                        //printf("jmin = %i\n", jmin);
                                        //printf("timediff = %i\n", tdiff);
					tdiffarray[i] = tdiff;
                                        ti_addm (ritarr, &imin, jtarr, &ier);
                                        ti_itoc (jtarr, valtimearray[i], &ier, 15);
			                strlength = strlen(valtimearray[i]);
					valtimearray[i][15] = '\0';
                                    }
				    timesinarray = att.len/2;
			            free(att.ivals);
				    proces = 1;
				    break;
				} else {
				    proces = 0;
				}
                            } else {
				proces = 0;
			    }
			}
		    }
		    if (proces == 0) {
			err = -5;
			er_wmsg("NC2GRID",&err," ",&iret,7,1);
		    }
		}

		//strncpy(parm,ncvar.parm,12);
                //printf ("parm = %s \n", parm);
		/* Replace -- in accum parm names as needed */
                //gb2_ctim ( tdiff,  parm  );
                //printf ("new parm = %s \n", parm);


		/* Allocate memory for pointers */

		if (proces == 1) {
		    coord = (char **)cmm_malloc2d ( nlevels, 5, sizeof(char), &iret );
                    ilevels1 = (int *)malloc(nlevels * sizeof(int));
                    ilevels2 = (int *)malloc(nlevels * sizeof(int));
		    nc_getvarlevels(ncid, nlevels, &var, nvars, varid, ilevels1, ilevels2, coord, &iret);
		    if (iret != 0) {
			err = -6;
			er_wmsg("NC2GRID",&err,var.name,&iret,7,strlen(var.name));
		        proces = 0;
		    }
		}
                if (proces == 1) {
                    /*
                     * Find Units Attributes 
                     */
                    for (attnum = 0; attnum < var.natts; attnum++) {
                        nc_status = nc_inq_attname (ncid, varid, attnum, att.name);
                        // Get variable attributes
                        nc_status = nc_inq_att (ncid, varid, att.name, &att.type, &att.len);
		        if (strcmp(att.name,"units") == 0) {
			    if (att.type <= NC_CHAR) {
                                att.string = (char *) malloc(att.len+1);
                                nc_status = nc_get_att_text (ncid, varid, att.name, att.string);
                                att.string[att.len] = '\0';

			        if (strcmp(att.string,"Pa") == 0 || strcmp(att.string,"Pascals") == 0 ) {
			            strcpy(units,"PA");
			        } else if (strcmp(att.string,"degree_C") == 0 ||
					   strcmp(att.string,"degC") == 0) {
			            strcpy(units,"C");
			        } else if (strcmp(att.string,"degree_F") == 0  || 
				           strcmp(att.string,"F") == 0 || 
					   strcmp(att.string,"degF") == 0) {
                                    strcpy(units,"F");
			        } else if (strcmp(att.string,"m") == 0 || 
                                           strcmp(att.string,"meters") == 0) {
                                    strcpy(units,"m");
			        } else if (strcmp(att.string,"%") == 0 ) {
                                    strcpy(units,"%");
			        } else if (strcmp(att.string,"m/s") == 0 ) {
                                    strcpy(units,"m/s");
			        } else if (strcmp(att.string,"1/s") == 0 ) {
                                    strcpy(units,"1/s");
			        } else if (strcmp(att.string,"K") == 0  ||
					   strcmp(att.string,"degree_K") == 0  || 
					   strcmp(att.string,"degK") == 0 ) {
                                    strcpy(units,"K");
			        }
			        free(att.string);
				break;
			    }
			}
                    } // end att loop
		    if (attnum == var.natts) {
			err = 2;
			er_wmsg("NC2GRID",&err,var.name,&iret,7,strlen(var.name));
		    }
		}
		if (proces == 1) {
		    /* Find other attributes for data transformation/unpacking (Mainly CDC reanalysis data) */
		    int foundscale = 0;
		    int found_offset = 0;
		    int found_missing = 0;
		    for (attnum = 0; attnum < var.natts; attnum++) {
                        nc_status = nc_inq_attname (ncid, varid, attnum, att.name);
                        // Get variable attributes
                        nc_status = nc_inq_att (ncid, varid, att.name, &att.type, &att.len);
                        if (strcmp(att.name,"scale_factor") == 0) {
			    if (att.type <= NC_FLOAT) {
				att.fvals = (float *) malloc(att.len * sizeof(float));
                                nc_status = nc_get_att_float (ncid, varid, att.name, att.fvals);
				scale_factor = att.fvals[0];
				foundscale = 1;
			    }
			} else if (strcmp(att.name,"add_offset") == 0) {
			    if (att.type <= NC_FLOAT) {
                                att.fvals = (float *) malloc(att.len * sizeof(float));
                                nc_status = nc_get_att_float (ncid, varid, att.name, att.fvals);
                                add_offset = att.fvals[0];
                                found_offset = 1;
                            }
			} else if (strcmp(att.name,"missing_value") == 0) {
                            if (att.type == NC_FLOAT) {
                                att.fvals = (float *) malloc(att.len * sizeof(float));
                                nc_status = nc_get_att_float (ncid, varid, att.name, att.fvals);
                                missing_value = att.fvals[0];
                                found_missing = 1;
                            } else if (att.type == NC_SHORT) {
				att.svals = (short *) malloc(att.len * sizeof(short));
				nc_status = nc_get_att_short (ncid, varid, att.name, att.svals);
				missing_value = (float)att.svals[0];
				found_missing = 1;
			    }
                        } else if (strcmp(att.name,"fillValue") == 0) {
                            if (att.type == NC_FLOAT) {
                                att.fvals = (float *) malloc(att.len * sizeof(float));
                                nc_status = nc_get_att_float (ncid, varid, att.name, att.fvals);
                                missing_value = att.fvals[0];
                                found_missing = 1;
                            } else if (att.type == NC_SHORT) {
                                att.svals = (short *) malloc(att.len * sizeof(short));
                                nc_status = nc_get_att_short (ncid, varid, att.name, att.svals);
                                missing_value = (float)att.svals[0];
                                found_missing = 1;
                            } else if (att.type == NC_DOUBLE) {
                                att.dvals = (double *) malloc(att.len * sizeof(double));
                                nc_status = nc_get_att_double (ncid, varid, att.name, att.dvals);
                                missing_value = (float)att.dvals[0];
                                found_missing = 1;
                            }

			}
		    }
		    if (foundscale == 0) {
			scale_factor = 1;
		    }
		    if (found_offset == 0) {
			add_offset = 0;
		    }
		    if (found_missing == 0) {
			missing_value = -9999;
		    } 
		    /*
		     * Copy gempak parameter from matching table entry to parm
		     * For temperature variables with '-', replace with units
		     */
		 //   char dash[]="-", *cptr;
		 //   cptr = strstr( ncvar.parm, dash);
                 //   //cptr = strstr( parm, dash);
		 //   if ( cptr != 0 ) {
		//	cst_rpst( ncvar.parm, dash, units, ncvar.parm, &ier);
		//	//cst_rpst( parm, dash, units, parm, &ier);
		//    }
		    
		//    //strncpy(parm,ncvar.parm,12);		    
		}
		/* 
		 * Determine if Variable has the attribute gridType 
		 * If gridType = Weather, print out the wxType ugly strings to files
		 */
		int wxtype = 0;

		for (attnum = 0; attnum < var.natts; attnum++) {
		    if (wxtype == 0) {
		        nc_status = nc_inq_attname (ncid, varid, attnum, att.name);
                        nc_status = nc_inq_att (ncid, varid, att.name, &att.type, &att.len);
                        if (strcmp(att.name,"gridType") == 0) {
			    if (att.type <= NC_CHAR) {
                                att.string = (char *) malloc(att.len+1);
                                nc_status = nc_get_att_text (ncid, varid, att.name, att.string);
                                att.string[att.len] = '\0';
			        if (strcmp(att.string, "WEATHER") == 0) {
				    wxtype = 1;
				    ighdr[0]=1;
                                } else if (strcmp(att.string, "DISCRETE") == 0) {
                                    wxtype = 2;
                                    ighdr[0]=1;
			        } else {
				    ighdr[0]=0;
				}
			    }
		        }
		    }
		}
		/* If the Variable is of gridType WEATHER, call function to extract wx ugly strings */
		 //printf("wxType = %i\n", wxtype );
		 //
		if (wxtype == 1) {
		    char wxstringname[40];
		    strcpy(wxstringname, var.name);
		    strcat(wxstringname, "_wxKeys");
		    nc_wxstrings (ncid, nvars, wxstringname, dims, valtimearray, &err);
		    if (err != 0) {
			err=-32;
                        er_wmsg("NC2GRID",&err," ",&iret,7,1);
		    }
	        } else if (wxtype == 2) {
		     char wxstringname[40];
                    strcpy(wxstringname, var.name);
                    strcat(wxstringname, "_keys");
                    nc_wxstrings (ncid, nvars, wxstringname, dims, valtimearray, &err);
                    if (err != 0) {
                        err=-32;
                        er_wmsg("NC2GRID",&err," ",&iret,7,1);
                    }
		}
		/*
		 * Open Grid file if not already open.
		 */
		if (proces == 1) {
		    if (igdfln == 0) {
			fill = 0;

			na_gcog (ui.gdoutf, ui.proj, ui.grdarea, ui.kxky, 
                                 ui.cpyfil, &fill, ui.garea, gdsarr, &ui.maxgrid, 
                                 &igdfln, rnvblk, &igrdnm, tmpproj, &igx, &igy, 
                                 &jx, &jy, ksbx, ksby, &subset, &err);
        		if (err != 0) {
        		    er_wmsg("NC2GRID",&err," ",&iret,7,1);
        		    break;
        		}
			igxold = rint( rnvblk[4] );
		    }
/* 
 *  If output gempak file grid has same navigation
 *  block as the current GRIB2 field, then unpack
 *  field and write to gempak file.
 */
                    navsiz = LLNNAV;
                    grc_cnav ( rnvblk, navblk, &navsiz, &gsflag, &err);
		}

		/*
		 * RETRIEVE GRID POINT VALUES FROM netCDF AND WRITE INDIVIDUAL GRIDS
		 * TO GEMPAK GRID FILE
		 */
		if (proces == 1 && gsflag == 1) {
		    //for (i = 0; i<nlevels; i++)
                    //     printf("levels1:  %i, levels2:  %i, coord = %s\n", ilevels1[i], ilevels2[i], (char *)coord[i]);

		    char *cvals[11], vcord[4];
		    char gdattim[21];
		    int k, ivcord;
		    float *grid;
		    int array_size = 1;

		    // get variable dimensions
		    for (k = 0; k < var.ndims; k++) {
                        array_size = array_size * dims[var.dims[k]].size;
                    }
		    var_val = (float *) malloc(array_size * sizeof(float));	
		    if (var.type == NC_FLOAT) {
		        nc_status = nc_get_var_float(ncid, varid, (float *)var_val);
                    } else if (var.type == NC_DOUBLE) {
                        /* Memory for both a short array and a float array */
                        dvar_val = (double *) malloc(array_size * sizeof(double));
                        nc_status = nc_get_var_double(ncid, varid, dvar_val);
                        for (k = 0; k < array_size;  k++) {
                            var_val[k] = (float)dvar_val[k];
                        }
                        free(dvar_val);
		    } else if (var.type == NC_SHORT) {
			/* Memory for both a short array and a float array */
                        svar_val = (short *) malloc(array_size * sizeof(short));
			nc_status = nc_get_var_short(ncid, varid, svar_val);
			for (k = 0; k < array_size;  k++) {
			    var_val[k] = (float)svar_val[k];
			}
			free(svar_val);
		    } else if (var.type == NC_BYTE) {
                        /* Variable data stored as a byte, convert to float */
			sc_val = (signed char *) malloc(array_size * sizeof(signed char));
			nc_status = nc_get_var_schar(ncid, varid, sc_val);
			for (k = 0; k < array_size;  k++) {
                            var_val[k] = (float)sc_val[k];
                        }
                        free(sc_val);
		    } else if (var.type == NC_INT) {
                        /* Variable data stored as a integer, convert to float */
                        ivar_val = (int *) malloc(array_size * sizeof(int));
                        nc_status = nc_get_var_int(ncid, varid, ivar_val);
                        for (k = 0; k < array_size;  k++) {
                            var_val[k] = (float)ivar_val[k];
                        }
                        free(ivar_val);
		    }
		    /*
		     * Split grids by time and level and write each grid to output GEMPAK grid file
		     */
		    // After using variable, clear memory
		    int xx=0;	// # grid points in grid counter
		    int ll=0;	// Level counter
		    int tt=0;   // Time counter
		    int cx=1;
		    int cy=1;
		    int gn;

		    /* Allocate memory for a single grid( 1 time, 1 level) */
		    grid = (float *) malloc(kx*ky * sizeof(float));
		    if ( grid == 0 ) {
                        err=-30;
                        er_wmsg("NC2GRID",&err," ",&iret,7,1);
                        continue;
		    }
		    /* Loop thru each data point and copy to grid array, write to file once kx*ky grid points have been found */
		    //printf ("Array Size = %i \n", array_size);
		    for (k = 0; k < array_size; k++) {
			if (ncfiletype == 3) {
			    gn = (kx*ky) - kx*cy + cx-1;
			    //gn = (kx*ky) - kx*cy + cx;
			    if (missing_value != -9999 && var_val[k] == missing_value) {
				grid[gn] = -9999;
			    } else {
			        grid[gn] = (var_val[k]*scale_factor)+add_offset;
			        grid[gn] = grid[gn] * ncvar.scalefactor;
                                //printf("(%i,%i), var_val[%i] = %f : grid[%i] = %f \n", cx, cy, k, var_val[k], gn, grid[gn]);

			    }
			    //if (cx == kx) {
			    if (cy == ky) {
				//cy = cy+1;
				//cx = 0;
				cy = 0;
                                cx = cx + 1;
			    }
			    //cx++;
			    cy++;
			} else {
			    if (missing_value != -9999 && var_val[k] == missing_value) {
				grid[xx] = -9999;
			    } else {
			        /* Apply scale factor and add Offset to unpack grid at each grid point */
			        grid[xx] = (var_val[k]*scale_factor)+add_offset;
			        /* Apply User defined sclae factor from table file */
			        grid[xx] = grid[xx] * ncvar.scalefactor;

			    }
			}
			xx++;	// Increment grid point counter
			if (xx == kx*ky) {
			    /* Maximum Number of grid points in grid reached, Write to file */
			    xx = 0;
			    cx = 1;
			    cy = 1;    
			    // Convert veritical coordinates
			    strncpy(vcord,coord[ll],4);
			    lv_cord (vcord, vparm, &ivcord, &iret, strlen(vcord), 12);
		            strncpy(gdattim, valtimearray[tt],15);
			    gdattim[15] = '\0';

			    //printf ("calktimearray = %s \n gdattim = %s \n", valtimearray[tt], gdattim);
/*
 *  Subset grid, if necessary.
 */
                            inx = igx;
                            iny = igy;
                            if ( subset == 1 ) na_gssg( &igxold, ksbx, ksby,
                                                        grid, &inx, &iny, &err );
                          //  printf(" ncvar.parm = %s\n", ncvar.parm);
			//	printf ("length of strignth = %i\n", strlen (ncvar.parm));
			    strncpy(parm,ncvar.parm,12);
			    //printf(" parm = %s\n", parm);
                            //printf(" ncvar.parm = %s\n", ncvar.parm);
			    //printf(" tdiff = %i tt = %i \n", tdiffarray[tt], tt);
			    
			    char dashdash[]="--", *cptrr;
			    cptrr = strstr( parm, dashdash);
                            if ( cptrr != 0 ) {
			        gb2_ctim ( tdiffarray[tt],  parm  );
                            }
			    //printf(" Post gb2_ctime  parm = %s\n", parm);
			    char dash[]="-", *cptr;
                            cptr = strstr( parm, dash);
                            if ( cptr != 0 ) {
                                cst_rpst( parm, dash, units, parm, &ier);
			    }
			    // Write grid to file
			    replace=ui.overwr;
                            printf("gdattim = %s \n", gdattim);
			    cgd_wpgd ( &igdfln, grid, &kx, &ky, ighdr,
                                           gdattim, "",
                                           &ilevels1[ll], &ilevels2[ll],
                                           &ivcord, parm, &replace,
                                           &pkmeth, &iprec, &err );
			    if ( err != 0 ) {
			        er_wmsg("GD",&err," ",&iret,2,1);
				if ( err == -11 && replace == 0 ) break;
			        continue;
			    }
			    prttl=0;
			    count++;
                            if( count % 50 == 1 ) prttl=1;
			    grc_wtrm (*fps, &prttl, &count, gdattim, " ", &ilevels1[ll], &ilevels2[ll], &ivcord, parm, &ier);
			    ll++;	// Increment to next level
			}
			/* Number of levels reached, increment to next time in file */
			if ( ll == nlevels ) {
			    tt++;	// Increment times
			    ll = 0;
			}
		    }
		    // Free allocated memory
		    free(grid);
		    free(var_val);
                    cmm_free2d ( (void **) coord, &iret );
                    free(ilevels1);
                    free(ilevels2);
		}
            } // end var loop
        }
/*
 * Close input netCDF file.
 */
	nc_status = nc_close(ncid);
/*
 *  Print summary.
 */
        //na_smry ( fps, &nfps, &count, &count, ui.ncfile, ui.gdoutf, &err );
/*
 * Close output
 */
        if ( fileflg == G_TRUE ) cfl_clos ( fp, &err );
    }
    /*
     *  Exit the GEMPAK user interface.
     */
    ip_exit( &iret );

}
