#include <netcdf.h>
#include "geminc.h"
#include "gemprm.h"

/*
 * nc_input struct contains all user input info.
 */
typedef struct {
    char ncfile[LLMXLN];          /*  input netCDF file name            */
    char gdoutf[LLMXLN];          /*  output GEMPAK file name           */
    char proj[LLMXLN];            /*  map projection information             */
    char grdarea[LLMXLN];         /*  grid area                              */
    char kxky[LLMXLN];            /*  size of grid                           */
    char maxgrd[LLMXLN];          /*  max number of grids                   **/
    char cpyfil[LLMXLN];          /*  grid navigation for gempak file       **/
    char garea[LLMXLN];           /*  graphics area                          */
    char output[LLMXLN];          /*  output devices                         */
    char nctbls[LLMXLN];          /*  list of GRIB2 tables                  **/
    int maxgrid;                  /*  integer version of maxgrd[]            */
    int  overwr;                  /*  Overwrite flag                         */
} nc_input;

typedef struct {                        /* dimension */
    char name[NC_MAX_NAME];
    size_t size;
} ncdim_t;

typedef struct {                        /* variable */
    char name[NC_MAX_NAME];
    nc_type type;
    int ndims;
    int *dims;
    int natts;
    int has_fillval;
    double fillval;
} ncvar_t;

typedef struct {                        /* attribute */
    int var;
    char name[NC_MAX_NAME];
    nc_type type;
    size_t len;
    char *string;               /* for text attributes (type = NC_CHAR) */
    double *dvals;              /* for numeric attributes ( type = NC_DOUBLE)*/
    float *fvals;		/* for numeric attributes ( type = NC_FLOAT)*/
    int *ivals;			/* for numeric attributes ( type = NC_INT)*/
    short *svals;		/* for numeric attributes ( type = NC_SHORT)*/

} ncatt_t;

typedef struct {                        /* global attribute */
    char name[NC_MAX_NAME];
    nc_type type;
    size_t size;
    char *string;
    double *dvals;
    float *fvals;
    int *ivals;

} ncgatt_t;

typedef struct {                        /* variable information */
    int nlevels;
    int *levels;			/* gempak levels */
    int *ivcord;			/* gempak vertical coordinates */
    char datetime[15];			/* date/time of variable record */
    char parm[8];			/* gempak paramter name */
} ncvarinfo_t;

#define NCVARS_TBL      "nctable.tbl"

/* NetCDF to GEMPAK table information */
typedef struct {
        char   ncparm[28];		/* netCDF Parm name	*/
	char   parm[12];			/* Gempak parm name */
	char   vcoord[10];
	float  scalefactor;
	int    timeflg;			/* timeflg - For GFE netCDF (which time to use in file)
					 * 1 - start Time, 0 - end time  */
} N2GVinfo;

typedef struct {
	int     nlines;         	/* number of lines         */
	N2GVinfo *info;          	/* grib2 table information        */
} N2Gvars_t;

void ncginp ( nc_input *, int * );
void nc_skvar ( char *ncparm, N2Gvars_t *ncvartbl, N2GVinfo *ncvar,
                  int *iret );
void nc_readvartbl ( char *tblname, N2Gvars_t *vartbl, int *iret );
void  nc_gtvartbl (char *nctbl, N2Gvars_t **ncvartbl, int *iret );
