#include <netcdf.h>
#include "geminc.h"
#include "gemprm.h"
#include "nc2grid.h"

void nc_getreftime (int ncid, int nvars, int ritarr[5], ncvar_t var, int *iret)

/************************************************************************
 * Search the netCDF file variables to find the reference time		*
 *									*
 * Input parameters:                                                    *
 * 	ncid		int		netCDF file number		*
 *      nvars		int		number variables in file	*
 *	var		ncvar_t		variable information		*
 *									*
 * Output parameters:							*
 *	reftime		char		gempak time (YYMMDD/HHmm)	*
 *	iret		int		return value			*
 *									*
 ************************************************************************/
{
	int varid, nc_status, attnum, ii, ier;
	int itarr[5];
	ncatt_t att;
	char tunits[4];
	double *dreftime;
	int foundreftime = 0;
	*iret = 0;

	// Search variables looking for time specific variables

	for (varid = 0; varid < nvars; varid++ ) {

	    // Find variabe attributes
	    nc_status = nc_inq_varndims(ncid, varid, &var.ndims);	    
            var.dims = (int *) malloc((var.ndims + 1) * sizeof(int));
            nc_status = nc_inq_var (ncid, varid, var.name, &var.type, &var.ndims, var.dims, &var.natts);
	    if (foundreftime == 0) {
	        // Look for variable named reference time
                if (strcmp (var.name, "reftime") == 0 || strcmp (var.name, "time") == 0 ) {
                
                    // Get Attributes of reftime or time Attributes
                    for (attnum = 0; attnum < var.natts; attnum++) {
                        nc_status = nc_inq_attname (ncid, varid, attnum, att.name);
			// Get attribute values

                        nc_status = nc_inq_att (ncid, varid, att.name, &att.type, &att.len);
                        // Get Attribute values
                        if ( strcmp(att.name,"units") == 0) {

                            if (att.type == NC_CHAR) {
                                att.string = (char *) malloc(att.len+1);
                                nc_status = nc_get_att_text (ncid, varid, att.name, att.string);
                                att.string[att.len] = '\0';
                                int ipos, n;

			        // get units and file reference time from unit att
                                char **cdp;
                                cdp = (char **)cmm_malloc2d ( 4, 16, sizeof(char), &ier );
                                // seperator string at space
                                cst_clst(att.string, ' ', " ", 4, 15, cdp, &n, &ier);
                                if (strcmp(cdp[0], "seconds") == 0 ) {
                                    strcpy(tunits, "SEC");
                                } else if (strcmp(cdp[0], "minutes") == 0 ) {
                                    strcpy(tunits, "MIN");
                                } else if (strcmp(cdp[0], "hours") == 0 ) {
                                    strcpy(tunits, "HRS");
                                } else if (strcmp(cdp[0], "days") == 0 ) {
                                    strcpy(tunits, "DAY");
                                }
                                if (strcmp(cdp[1], "since") == 0 ) {
                                    char **datearr;
                                    datearr = (char **)cmm_malloc2d ( 3, 11, sizeof(char), &ier );
                                    int o;
                                    cst_clst(cdp[2], '-', " ", 3, 10, datearr, &o, &ier);
                                    unsigned char c;
                                    if (strlen(datearr[0])==5) {
                                        for (ii=1; ii<strlen(datearr[0]); ii++) {
                                            c = datearr[0][ii];
                                            datearr[0][ii-1] = c;
                                        }
                                        datearr[0][4] = '\0';
                                    }
                                    cst_numb(datearr[0], &itarr[0], &ier);
                                    cst_numb(datearr[1], &itarr[1], &ier);
                                    cst_numb(datearr[2], &itarr[2], &ier);
				    cmm_free2d ( (void **) datearr, &ier );
                                }
                                if (n == 4) {
                                    char **timearray;
                                    timearray = (char **)cmm_malloc2d ( 3, 11, sizeof(char), &ier );
                                    // seperator string at space
                                    int p;
                                    cst_clst(cdp[3], ':', " ", 3, 10, timearray, &p, &ier);
                                    cst_numb(timearray[0], &itarr[3], &ier);
                                    cst_numb(timearray[1], &itarr[4], &ier);
                                    cmm_free2d ( (void **) timearray, &ier );
                                } else {
                                    itarr[3] = 0;
                                    itarr[4] = 0;
                                }
			        cmm_free2d ( (void **) cdp, &ier );
                            }
                        }
                    }
		    ritarr[0] = itarr[0];
		    ritarr[1] = itarr[1];
		    ritarr[2] = itarr[2];
		    ritarr[3] = itarr[3];
		    ritarr[4] = itarr[4];
                    int imin;
		    /* Only retrieve variable data if variable name is reftime */
		    if (strcmp (var.name, "reftime") == 0) {
                        if (var.type == NC_DOUBLE) {
                            // get variable dimensions
                            dreftime = (double *) malloc((1) * sizeof(double));
                            nc_status = nc_get_var_double(ncid, varid, dreftime);
                            imin =  (int)(dreftime[0]/60);
                            free(dreftime);
                        }
                        ti_addm_ (itarr, &imin, itarr, &ier);
			ritarr[0] = itarr[0];
                        ritarr[1] = itarr[1];
                        ritarr[2] = itarr[2];
                        ritarr[3] = itarr[3];
                        ritarr[4] = itarr[4];
			*iret = 1;
		    } else {
			*iret = 0;
		    }
			foundreftime = 1;
                } 
	    }
	}
	free(var.dims);
	if (foundreftime == 0) {
	    /* All AWIPS and GFE netCDF Files use 1/1/1970 as the referance time */
	    ritarr[0] = 1970;
            ritarr[1] = 1;
            ritarr[2] = 1;
            ritarr[3] = 0;
            ritarr[4] = 0;
	    *iret = -1;
	}
}
