#include <netcdf.h>
#include <string.h>
#include "geminc.h"
#include "gemprm.h"
#include "nc2grid.h"

void nc_getvalidtimes (int ncid, ncdim_t *dims, int nvars, int ritarr[5], ncvar_t var, int timeflg, char valtimearray[][16], int *iret)

/************************************************************************
 * Search the netCDF file variables to find the valid times		*
 *									*
 * Input parameters:                                                    *
 * 	ncid		int		netCDF file number		*
 *      dims            ncdim_t         file diminsions definitions	*
 *      nvars		int		number variables in file	*
 *      reftime         char		reference time (YYMMDD/HHmm)	*
 *	var		ncvar_t		variable information		*
 *      timeflg		int		time format flag		*
 *						0 - valid time		*
 *						1 - reftime and fhr     *
 *									*
 * Output parameters:							*
 *      valtimearray	char		array of gempak valid times	*
 *	iret		int		return value			*
 *									*
 ************************************************************************/
{
	int varid, nc_status, attnum, ii, imin, ihrs, ier, lens;
	int itarr[5], jtarr[5];
	int foundvalidtimes = 0;
	ncatt_t att;
	char tunits[4], temp_valtime[15];
	*iret = 0;
	int *ivaltimes;
	double *dvaltimes;
	char **fhrs;
	char tempstring[15];

	// Search variables looking for time specific variables

	for (varid = 0; varid < nvars; varid++ ) {

	    // Find variable attributes
	    nc_status = nc_inq_varndims(ncid, varid, &var.ndims);	    
            var.dims = (int *) malloc((var.ndims + 1) * sizeof(int));
            nc_status = nc_inq_var (ncid, varid, var.name, &var.type, &var.ndims, var.dims, &var.natts);

            if (strcmp (var.name, "valtimeMINUSreftime") == 0 ) {
                if (var.ndims == 1) {
                    if (var.type == NC_INT) {
                        ivaltimes = (int *)malloc(dims[var.dims[0]].size * sizeof(int));
                        nc_status = nc_get_var_int(ncid, varid, ivaltimes);
                    }
                }
                fhrs = (char **)cmm_malloc2d ( dims[var.dims[0]].size, 3, sizeof(char), &ier );
                for (ii = 0; ii < dims[var.dims[0]].size; ii++) {
                    imin = (int)(ivaltimes[ii]/60);
                    ihrs = imin/60;

		    if (timeflg == 1) {
                        if (ihrs < 10) sprintf(fhrs[ii], "00%d", ihrs);
                        if (ihrs >=10 && ihrs < 100) sprintf(fhrs[ii], "0%d", ihrs);
                        if (ihrs >=100) sprintf(fhrs[ii], "%d", ihrs);
			ti_itoc (ritarr, valtimearray[ii], &ier, 15);
			valtimearray[ii][11] = '\0';
                        strcat(valtimearray[ii], "F");
                        strcat(valtimearray[ii],fhrs[ii]);
		    } else {
                        ti_addm (ritarr, &imin, jtarr, &ier);
                        ti_itoc (jtarr, temp_valtime, &ier, 15);
			strncpy(valtimearray[ii],temp_valtime,11);
			
		    }
                }
		foundvalidtimes = 1;
	    } else if ( strcmp (var.name, "time") == 0 ) {
		/* CDC Renalysis netCDF */
		printf("car.ndims = %i\n", var.ndims);
		if (var.ndims == 1) {
                    if (var.type == NC_DOUBLE) {
                        dvaltimes = (double *)malloc(dims[var.dims[0]].size * sizeof(double));
                        nc_status = nc_get_var_double(ncid, varid, dvaltimes);
                    } 
		    for (ii = 0; ii < dims[var.dims[0]].size; ii++) {
                        ihrs = (int)(dvaltimes[ii]);
                        imin = ihrs*60;
		        if (timeflg == 1) {
                            if (ihrs < 10) sprintf(fhrs[ii], "00%d", ihrs);
                            if (ihrs >=10 && ihrs < 100) sprintf(fhrs[ii], "0%d", ihrs);
                            if (ihrs >=100) sprintf(fhrs[ii], "%d", ihrs);
			    ti_itoc (ritarr, valtimearray[ii], &ier, (FDTTMS_SIZE - 1));
                            strcat(valtimearray[ii], "F");
                            strcat(valtimearray[ii],fhrs[ii]);
                        } else {
                            ti_addm (ritarr, &imin, jtarr, &ier);
                            ti_itoc (jtarr, temp_valtime, &ier, (FDTTMS_SIZE - 1));
			    st_null ( temp_valtime, temp_valtime, &lens, &ier, sizeof(temp_valtime), sizeof(temp_valtime));
			    strncpy(valtimearray[ii],temp_valtime,sizeof(temp_valtime));
                        }
		    }
                } else if (var.ndims == 2) {
                    if (var.type == NC_CHAR) {
                        char *timestring;

                        timestring = (char *)malloc( dims[var.dims[1]].size +1 );
                        nc_status = nc_get_var_text (ncid, varid, timestring );
                        printf("TIME STRING = %s \n", timestring);

                        char yy[10] = "\0";
                        char mm[10] = "\0";
                        char dd[10] = "\0";
                        char hh[10] = "\0";
                        strncpy(yy,timestring+2,2);
                        
                        strncpy(mm,timestring+5,2);
                        strcat(mm,"\0");

                        strncpy(dd,timestring+8,2);
                        strcat(dd,"\0");

                        strncpy(hh,timestring+11,2);
                        strcat(hh,"\0");
                        ii=0;
			sprintf(valtimearray[ii], "%s%s%s/%s00", yy, mm, dd, hh);
                       printf("ii = %i, %s \n", ii, valtimearray[ii]);
                    }
                }
		printf("Found Valid Time \n");
		foundvalidtimes = 1;
            } else if ( strcmp (var.name, "timeofdata") == 0 ) {
                if (var.type <= NC_CHAR) {
                    nc_status = nc_get_var_text(ncid, varid, temp_valtime);
                    st_null ( temp_valtime, temp_valtime, &lens, &ier, sizeof(temp_valtime), sizeof(temp_valtime));
                    int x;
                    for ( x = 2; x < 8; ++x ) {
                        tempstring[x-2] = temp_valtime[x];
                    }
                    strcat(tempstring,"/1200F000");
                    strcpy(valtimearray[0],tempstring);
                    foundvalidtimes = 1;
                }
            } else {
         //       printf (" No variable named valtimeMINUSreftime in file \n");
	//	*iret = 1;
            }
		printf("Freeing memory/n");
	    free(var.dims);
        }
	printf ("foundvalidtimes = %i \n", foundvalidtimes);
	if (foundvalidtimes == 0) {
	    *iret = 1;
	}
}
