#include <string.h>
#include <netcdf.h>
#include "geminc.h"
#include "gemprm.h"
#include "nc2grid.h"

void nc_getvarlevels (int ncid, int nlevels, ncvar_t *var, int nvars, int varid,  
                      int *ilevels1, int *ilevels2, char **coord,
                      int *iret)

/************************************************************************
 * Search the Variable For the vertical levels				*
 *									*
 * Input parameters:                                                    *
 * 	ncid		int		netCDF file number		*
 *      nlevels         int             Number of Levels for variable	*
 *      var             ncvar_t         varaible definitions		*
 *      nvars           int		number of variables in file	*
 *      varid           int             variable ID number              *
 *									*
 * Output parameters:							*
 *	*ilevel1  	int		array of veritcal level 1       *
 *	*ilevel2        int             array of veritcal level 2       *
 *      coord[]		char		Vertical coordinites		*
 *	iret		int		return value			*
 *					    -1 - No levels found	*
 ************************************************************************/
{
    char **levels;
    int tvarid, attnum;
    int num_lev, ii;
    int levcnt = 0;
    int icnt, found, nc_status, ier;
    char lastcoord[5];
    float *var_val;
    ncvar_t tvar;	// Internal ncvar_t structure
    ncatt_t att;
/*------------------------------------------------------------------------*/
    found = 0;

/*
 * Search the passed variable's attributes to find level information.
 */
    for (attnum = 0; attnum < var->natts; attnum++) {
	nc_status = nc_inq_attname (ncid, varid, attnum, att.name);
        // Get variable attributes
        nc_status = nc_inq_att (ncid, varid, att.name, &att.type, &att.len);
	// allocate memory for 2-D Character array
        levels = (char **)cmm_malloc2d ( 40, 11, sizeof(char), &ier );

        if (strcmp(att.name,"levels") == 0 || strcmp(att.name,"level") == 0) {
	    if (att.type <= NC_CHAR) {
                // Retrieve level string from file
                att.string = (char *) malloc(att.len+1);
                nc_status = nc_get_att_text (ncid, varid, att.name, att.string);
                att.string[att.len] = '\0';

                // Split level string into array of strings
                cst_clst(att.string, ' ', " ", 40, 10, levels, &num_lev, &ier);
                free(att.string);

                levcnt = 0;
                // Convert string array into array of levels and coord
	        for (icnt = 0; icnt < num_lev; icnt++ ) {
		    if (levcnt == nlevels) {
			break;
		    }
                    if (strcmp(levels[icnt],"MB") == 0 ) {
                        strcpy(lastcoord,"PRES\0");
                    } else if (strcmp(levels[icnt],"FHAG") == 0 ) {
                        strcpy(lastcoord,"HGHT\0");
                    } else if (strcmp(levels[icnt],"BL") == 0 ) {
                        strcpy(lastcoord,"PDLY\0");
                    } else if (strcmp(levels[icnt],"SIG") == 0 ) {
                        strcpy(lastcoord,"SGMA\0");
                    } else if (strcmp(levels[icnt],"TROP") == 0 ) {
                        strcpy(lastcoord,"TROP\0");
                        strcpy(coord[levcnt],lastcoord);
                        ilevels1[levcnt] = 0;
                        ilevels2[levcnt] = -1;
                        levcnt++;
                    } else if (strcmp(levels[icnt],"SFC") == 0 ||
                               strcmp(levels[icnt],"MSL") == 0 ||
                               strcmp(levels[icnt],"EA") == 0) {
                        strcpy(lastcoord,"NONE\0");
                        strcpy(coord[levcnt],lastcoord);
                        ilevels1[levcnt] = 0;
                        ilevels2[levcnt] = -1;
                        levcnt++;
                    } else {
                        int nout;
                        char **split;
                        split = (char **)cmm_malloc2d ( 2, 11, sizeof(char), &ier );
                        cst_clst(levels[icnt], '>', " ", 2, 10, split, &nout, &ier);
                        if (nout == 2) {
                            if (strcmp(lastcoord,"SGMA\0") == 0) {
                                cst_numb(split[1], &ilevels2[levcnt], &ier);
                                cst_numb(split[0], &ilevels1[levcnt], &ier);
                                ilevels1[levcnt] = ilevels1[levcnt]*100;
                                ilevels2[levcnt] = ilevels2[levcnt]*100;
                            } else {
                                cst_numb(split[1], &ilevels1[levcnt], &ier);
                                cst_numb(split[0], &ilevels2[levcnt], &ier);
                            }
                        } else {
                            cst_numb(split[0], &ilevels1[levcnt], &ier);
                            ilevels2[levcnt] = -1;
                        }
                        strcpy(coord[levcnt],lastcoord);
                        levcnt++;
                        cmm_free2d ( (void **) split, &ier );
                    }
                }
            }
	    found = 1;
            break;
        } else if (strcmp(att.name,"level_desc") == 0 ) {
            if (att.type <= NC_CHAR) {
                // Retrieve level string from file
                att.string = (char *) malloc(att.len+1);
                nc_status = nc_get_att_text (ncid, varid, att.name, att.string);
                att.string[att.len] = '\0';
                // Split level string into array of strings
                cst_clst(att.string, ' ', " ", 40, 10, levels, &num_lev, &ier);
                free(att.string);
                levcnt = 0;

                if (num_lev == 2) {
                    if (strcmp(levels[1],"m") == 0) {
                        strcpy(coord[0],"HGHT");
                    }
                    cst_numb(levels[0], &ilevels1[0], &ier);
                    ilevels2[0] = -1;
                }
            }
	    found = 1;
            break;
	}
	cmm_free2d ( (void **) levels, &ier );
    }
/*
 * Search for a level variable and get vertical levels from variable and
 * Variable attributes.
 */
    for (tvarid = 0; tvarid < nvars; tvarid++ ) {
	nc_status = nc_inq_varndims(ncid, tvarid, &tvar.ndims);
        tvar.dims = (int *) malloc((tvar.ndims + 1) * sizeof(int));
        nc_status = nc_inq_var (ncid, tvarid, tvar.name, &tvar.type, 0, tvar.dims, &tvar.natts);
	/* Find Level Variable, if exists, get levels*/
	if (strcmp(tvar.name, "level") == 0 ) {
	    if (tvar.type == NC_FLOAT) {
		var_val = (float *) malloc(nlevels * sizeof(float));
		nc_status = nc_get_var_float(ncid, tvarid, var_val);
	    }
	    /* save levels as integers in ilevel1 array */
	    for (ii = 0; ii < nlevels; ii++) {
		ilevels1[ii] = (int)var_val[ii];
		ilevels2[ii] = -1;
	    }
	    /* free memory*/
	    free(var_val);
	    /* find Attributes to determine vertical coordinate system */
	    for (attnum = 0; attnum < tvar.natts; attnum++) {
		nc_status = nc_inq_attname (ncid, tvarid, attnum, att.name);
		nc_status = nc_inq_att (ncid, tvarid, att.name, &att.type, &att.len);
                if (strcmp(att.name,"units") == 0) {
		    if (att.type <= NC_CHAR) {
                        att.string = (char *) malloc(att.len+1);
                        nc_status = nc_get_att_text (ncid, tvarid, att.name, att.string);
                        att.string[att.len] = '\0';
		        if (strcmp(att.string,"millibar") == 0 ) {
		            strcpy(lastcoord,"PRES\0");
		        }
		    }
		    for (ii = 0; ii < nlevels; ii++) {
			strcpy(coord[ii], lastcoord);
		    }
		}
	    }
	    found = 1;
	    break;
	}
    }

/* 
 * If nlevels = 1 and levels are not found in variable attributes, set level to default
 */ 
    if (found == 0) {
        if(nlevels == 1) {
            ilevels1[0] = 0;
            ilevels2[0] = -1;
            strcpy(coord[0],"NONE");
        } else {
            *iret = -1;
        }
    }
}
