#include "nc2grid.h"

void  nc_gtvartbl (char *nctbl, N2Gvars_t **ncvartbl, int *iret )
/************************************************************************
 * nc_gtvartbl                                                          *
 *                                                                      *
 * This ruitine will read in a NC2GEMPAK parameter table from a         *
 * specified file and returns a structure containing table entries.	*
 *                                                                      *
 * nc_gtvartbl (nctbl, ncvartbl, iret )                                 *
 *                                                                      *
 * Input parameters:                                                    *
 *      *nctbl          char            Filename of the table to read   *
 *                                                                      *
 * Output parameters:                                                   *
 *      *ncvartbl       N2Gvars_t       Pointer to list of table entries*
 *      *iret           int             Return code                     *
 *                                       -31 = Error reading table      *
 **                                                                     *
 * Log:                                                                 *
 * C. Bailey/HPC        09/09   Creation                                *
 ***********************************************************************/
{
    int ier;
    char tmpname[LLMXLN];
    static char currtable[LLMXLN];
    static N2Gvars_t currvartbl={0,0};

/*---------------------------------------------------------------------*/
    *iret = 0;

    /*
     *  Check if user supplied table.  If not, use default.
     */
    if ( strlen(nctbl) == (size_t)0 ) {
        strcpy( tmpname,"nctable.tbl");
    }
    else {
        strcpy( tmpname, nctbl );
    }

    /*
     *  Check if table has already been read in. 
     *  If different table, read new one in.
     */
    if ( strcmp(tmpname, currtable ) != 0 ) {
        if ( currvartbl.info != 0 ) {
            free(currvartbl.info);
            currvartbl.info=0;
            currvartbl.nlines=0;
        }
        printf(" Opening NetCDF Parameter Table %s...\n", tmpname);
	nc_readvartbl ( tmpname, &currvartbl, &ier );
	if ( ier != 0 ) {
            currvartbl.nlines=0;
            *iret=-31;
            *ncvartbl = &currvartbl;
            return;
        }
    }
    strcpy( currtable, tmpname );
    *ncvartbl = &currvartbl;

}
