#include "geminc.h"
#include "gemprm.h"
#include "nc2grid.h"

void nc_readvartbl ( char *tblname, N2Gvars_t *vartbl, int *iret )
/************************************************************************
 * nc_readvartbl							*
 *									*
 * This ruitine will read in a NC2GEMPAK parameter table into an array	*
 * of structures.							*
 *									*
 * nc_readvartbl (tbname, vartbl, iret )				*
 *									*
 * Input parameters:							*
 *	*tbname		char		Filename of the table to read	*
 *									*
 * Output parameters:							*
 *	*vartbl		N2Gvars_t	Pointer to list of table entries*
 *	*iret		int		Return code			*
 *					  0 = Successful		*
 *					 -1 = Could not open		*
 *					 -2 = Could not get count of	*
 *					      of table entries. 	*
 **									*
 * Log:									*
 * C. Bailey/HPC	09/09	Creation				*
 ***********************************************************************/
{
	FILE     *fp;
	int      n, blen, nr, ier;
	char     buffer[256];
	char	nparm[28], gparm[13], vcord[10], cscale[11], timeflg[2];
	int     itimeflg;
	float	scale;

/*---------------------------------------------------------------------*/
        *iret = G_NORMAL;

        /*
         *  Open the table. If not found return an error.
         */

        fp = cfl_tbop( tblname, "grid", &ier);
        if ( fp == NULL  ||  ier != 0 )  {
            *iret = -1;
            return;
        }

	cfl_tbnr(fp, &nr, &ier);
        if ( ier != 0 || nr == 0 ) {
            *iret = -2;
            cfl_clos(fp, &ier);
            return;
        }

	printf("number of lines in fil = %i size of %i\n", nr,sizeof(N2GVinfo));
	vartbl->info = (N2GVinfo *)malloc((size_t)nr*sizeof(N2GVinfo));
        vartbl->nlines = nr;

	n  = 0;
        while ( n < nr ) {

            cfl_trln( fp, 256, buffer, &ier );
            if ( ier != 0 ) break;

	    /* Initialize strings with all blank characters */
	    nparm[0] = '\0';
	    gparm[0] = '\0';
            vcord[0] = '\0';
	    cscale[0] = '\0';
	    timeflg[0] = '\0';

            sscanf( buffer, "%27c %12c %9c %10c %c",
                            nparm, gparm, vcord, cscale, timeflg );
	    nparm[27] = '\0';
	    gparm[12] = '\0';
	    vcord[9] = '\0';
	    cscale[10] = '\0';
	    timeflg[1] = '\0';
	    scale = (float)atof(cscale);
	    itimeflg = (int)atoi(timeflg);
	    
	    strcpy(vartbl->info[n].ncparm, nparm);
	    strcpy(vartbl->info[n].parm, gparm);
	    strcpy(vartbl->info[n].vcoord, vcord);
	    vartbl->info[n].scalefactor = scale;
	    vartbl->info[n].timeflg = itimeflg;

	    strcpy(cscale,"          ");
	    strcpy(timeflg," ");

	    n++;
	}
	cfl_clos(fp, &ier);

}
