#include "nc2grid.h"

void     nc_skvar(char *ncparm, N2Gvars_t *ncvartbl, N2GVinfo *ncvar, 
                  int *iret)
/************************************************************************
 * nc_skvar                                                             *
 *									*
 * This function searches a netCDF parameter table from a               *
 * N2Gvars_t structure, and returns a structure containing the table    *
 * elements for the entry that matches the netCDF parmater name.	*
 *									*
 * nc_skvar(ncparm, ncvartbl, ncvar, iret)				*
 *									*
 * Input parameters: 							*
 *	ncparm		char		netCDF parameter name		*
 *	ncvartbl	N2Gvars_t	structure containing NC table	*
 *					  entries			*
 *									*
 * Output parameters: 							*
 *	ncvar		N2GVinfo	structure for parameter table	*
 *					   entry			*
 *	iret		int		Return code			*
 *                                        0 = entry found               *
 *                                       -1 = entry NOT found           *
 **                                                                     *
 * Log:                                                                 *
 * C. Bailey/HPC	09/2009		                                *
 ***********************************************************************/
{
    char temp[27];
    char blanks[27]="                           ";
    int n, slen;
/*---------------------------------------------------------------------*/
    *iret = -1;

    slen = strlen(ncparm);
    strcpy(temp,ncparm);
    if (slen > 27) temp[27] = '\0';
    if (slen < 27) {
        strncat(temp, blanks, 27-slen );
    }

    n=0;
	//printf("%i \n", ncvartbl->nlines);
    while ( n < ncvartbl->nlines ) {
	 //printf("%s:%s| %i\n", temp, ncvartbl->info[n].ncparm, strlen(ncvartbl->info[n].ncparm));
	if ( strcmp(temp,ncvartbl->info[n].ncparm) == 0) {

	    strcpy(ncvar->ncparm,ncvartbl->info[n].ncparm);
	    strcpy(ncvar->parm,ncvartbl->info[n].parm);
	    strcpy(ncvar->vcoord,ncvartbl->info[n].vcoord);
	    ncvar->scalefactor = ncvartbl->info[n].scalefactor;
	    ncvar->timeflg = ncvartbl->info[n].timeflg;
	    *iret=0;
	     break;
        }
        n++;
    }

}
