#include <netcdf.h>
#include "geminc.h"
#include "gemprm.h"
#include "nc2grid.h"

void nc_wxstrings (int ncid, int nvars, char *wxstringname, ncdim_t *dims, char valtimearray[][16], int *iret)

/************************************************************************
 * Search the netCDF file variables to find the Weather Ugly Strings    *
 * Will Print strings to a file for each forecast time.                 *
 *									*
 * Input parameters:                                                    *
 * 	ncid		int		netCDF file number		*
 *      nvars		int		number variables in file	*
 *	wxstringname    char		name of netCDF wx string name   *
 *      dims            ncdim_t         file diminsions definitions     *
 *	valtimearray    char		array of valid times		*
 *									*
 * Output parameters:							*
 *	iret		int		return value			*
 *					 -1 - Could not find wxKeys	*
 *									*
 ************************************************************************/
{
	int varid, nc_status, ii, jj, ier;
	int ntimes, nstrings, maxlen;
	int nels = 1;
	int wxfound = 0;
	ncvar_t var;
        FILE *fptr;
        char filename[100];

	*iret = 0;
	char *uglystrings;
	
	printf ("Entering nc_wxstring\n");
	/*
	 * Search netCDF file for _wxKeys variable.
	 */
	for (varid = 0; varid < nvars; varid++ ) {

	    // Find variable attributes
	    nc_status = nc_inq_varndims(ncid, varid, &var.ndims);	    
            var.dims = (int *) malloc((var.ndims + 1) * sizeof(int));
            nc_status = nc_inq_var (ncid, varid, var.name, &var.type, &var.ndims, var.dims, &var.natts);
            if (strcmp (var.name, wxstringname) == 0 ) {
                /* Make sure grid demensions are 3 */
		if (var.ndims == 3) {
		    ntimes = dims[var.dims[0]].size;
		    nstrings = dims[var.dims[1]].size;
		    maxlen = dims[var.dims[2]].size;
		    nels = ntimes*nstrings*maxlen;
		    /* Allocate Memory for Character Pointer */
		    uglystrings = (char *)malloc( nels+1 );
		    nc_status = nc_get_var_text(ncid, varid, uglystrings);
		    int cnt = 0, ipos = 15;
		    const char *sp;
		    unsigned char uc;
                    ii=0;
		    printf ("ntmes = %i \n", ntimes);
		    char tempvaltime[16];
		    for (ii == 0; ii<ntimes; ii++) {
			/* Create File Name for output file using validtime */
			strncpy(tempvaltime,valtimearray[ii],16);
			int strlength = strlen(tempvaltime);
			jj=0;
			for (jj == 0; jj<strlength; jj++) {
			    uc = tempvaltime[jj];
			    if (uc == 32) {
				ipos = jj;
				break;
			    }
			}
			tempvaltime[ipos] = '\0';
			if (strcmp (wxstringname,"Wx_SFC_wxKeys") == 0 ) {
			    strcpy(filename, "wxKeys_");
                        } else {
			    strcpy(filename, wxstringname);
			    strcat(filename, "_");
                        }
			strcat(filename, tempvaltime);
			strcat(filename,".txt");
			cst_rpst(filename, "/", "_", filename, &ier);
			printf ("filename = %s\n", filename);
			/* Open output file */
			fptr = fopen(filename,"w");

			jj = 0;
			for (jj == 0; jj<nstrings; jj++) {
			    sp = uglystrings+(cnt*maxlen);
			    cnt++;
			    fprintf(fptr, "%3i = %s\n", jj, sp);
			}
			/* close output file */
			ier = fclose(fptr);
		    }
		    wxfound = 1;
		    return;
                } else {
		    *iret = -2;
		}
	        free(uglystrings);
	    } 
	}
	if (wxfound == 0) {
	    *iret = -1;
	}
}
