#include "nc2grid.h"

void ncginp( nc_input *input, int *iret )
/************************************************************************
 * ncginp                                                               *
 *                                                                      *
 * This routine reads the user input variables for nagrib2.             *
 *                                                                      *
 *  Usage:                                                              *
 *       int = nag2in( input, iret );                                   *
 *                                                                      *
 *                                                                      *
 *  Output Arguments:                                                   *
 *    *input        nc_input            Structure to hold user input    *
 *    *iret         int                 Error return                    *
 *                                      0  = Successfull                *
 *                                      -2 = Error getting one or more  *
 *                                           variables                  *
 *                                                                      *
 * Log:                                                                 *
 * C. Bailey/NCEP           08/2009                                     *
 ***********************************************************************/
{
    int ret1,ret2,ret3,ret4,ret5,ret6,ret7,ret8,ret9,ret10,ret11;
    int sumret, len, num, i;
    static int ivarlen=8;   
    char tmpstr[10];
    char *cptr[2];
    char tmpout[LLMXLN], tmpout2[LLMXLN];

    char ncfile[] ="NCFILE  ";
    char gdoutf[] ="GDOUTF  ";
    char proj[]   ="PROJ    ";
    char grdarea[]="GRDAREA ";
    char kxky[]   ="KXKY    ";
    char maxgrd[] ="MAXGRD  ";
    char cpyfil[] ="CPYFIL  ";
    char garea[]  ="GAREA   ";
    char output[] ="OUTPUT  ";
    char nctbls[] ="NCTBLS  ";
    char ovrwrt[] ="OVERWR  ";

    /*
     *  Get NCFILE
     */
    memset(input->ncfile,0,LLMXLN);
    ip_str( ncfile, input->ncfile, &ret1, ivarlen, LLMXLN);
    st_null(input->ncfile, input->ncfile, &len, iret, LLMXLN, LLMXLN);

    /*
     *  Get GDOUTF
     */
    memset(input->gdoutf,0,LLMXLN);
    ip_str( gdoutf, input->gdoutf, &ret2,ivarlen,LLMXLN);
    st_null(input->gdoutf, input->gdoutf, &len, iret, LLMXLN, LLMXLN);
    cst_lcuc( input->gdoutf, tmpout, &ret2 );

    /*
     *  Get PROJ
     */
    memset(input->proj,0,LLMXLN);
    ip_str( proj, input->proj, &ret3,ivarlen,LLMXLN);
    st_null(input->proj, input->proj, &len, iret, LLMXLN, LLMXLN);

    /*
     *  Get GRDAREA
     */
    memset(input->grdarea,0,LLMXLN);
    ip_str( grdarea, input->grdarea, &ret4,ivarlen,LLMXLN);
    st_null(input->grdarea, input->grdarea, &len, iret, LLMXLN, LLMXLN);

    /*
     *  Get KXKY
     */
    memset(input->kxky,0,LLMXLN);
    ip_str( kxky, input->kxky, &ret5,ivarlen,LLMXLN);
    st_null(input->kxky, input->kxky, &len, iret, LLMXLN, LLMXLN);

    /*
     *  Get MAXGRD
     */
    memset(input->maxgrd,0,LLMXLN);
    ip_str( maxgrd, input->maxgrd, &ret6,ivarlen,LLMXLN);
    st_null(input->maxgrd, input->maxgrd, &len, iret, LLMXLN, LLMXLN);
    st_numb(input->maxgrd, &(input->maxgrid), iret, len);
    if ( input->maxgrid >= MMHDRS ) {
        input->maxgrid = MMHDRS-1;
        cst_inch (input->maxgrid, tmpstr, iret );
        *iret=6;
        er_wmsg("NC2GRID",iret,tmpstr,&num,7,10);
    }

    /*
     *  Get CPYFIL
     */
    memset(input->cpyfil,0,LLMXLN);
    ip_str( cpyfil, input->cpyfil, &ret7,ivarlen,LLMXLN);
    st_null(input->cpyfil, input->cpyfil, &len, iret, LLMXLN, LLMXLN );

    /*
     *  Split cpyfil using "|".  If optional subarea exists,
     *  set garea to it.  Otherwise get garea from dynamic tutor.
     */
    cptr[0]=tmpout;
    cptr[1]=tmpout2;
    cst_clst(input->cpyfil, '|', "\0", 2, LLMXLN, cptr, &num, iret);
    strncpy(input->cpyfil, cptr[0], (size_t)LLMXLN);
    if ( num == 2 ) {
        strncpy(input->garea, cptr[1], (size_t)LLMXLN);
    }
    else {
        memset(input->garea,0,LLMXLN);
        ip_str( garea, input->garea, &ret8,ivarlen,LLMXLN);
        st_null(input->garea, input->garea, &len, iret, LLMXLN, LLMXLN);
    }

    /*
     *  Get OUTPUT
     */
    memset(input->output,0,LLMXLN);
    ip_str( output, input->output, &ret9,ivarlen,LLMXLN);
    st_null(input->output, input->output, &len, iret, LLMXLN, LLMXLN);

    /*
     *  Get NCTBLS
     */
    memset(input->nctbls,0,LLMXLN);
    ip_str( nctbls, input->nctbls, &ret10, ivarlen, LLMXLN);
    st_null(input->nctbls, input->nctbls, &len, iret, LLMXLN, LLMXLN);

    /*
     *  Get OVERWR
     */
    ip_log( ovrwrt, &(input->overwr), &ret11,ivarlen);


    sumret=ret1+ret2+ret3+ret4+ret5+ret6+ret7+ret8+ret9+ret10+ret11;

    if ( sumret == 0 ) *iret = 0;
    else *iret = -2;

}

